'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = accordion;

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _raf = require('raf');

var _raf2 = _interopRequireDefault(_raf);

var _grid = require('./grid');

var _grid2 = _interopRequireDefault(_grid);

var _column = require('./column/column');

var _column2 = _interopRequireDefault(_column);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function accordion(pure) {
  var React = pure.React,
      PropTypes = pure.PropTypes,
      cloneElement = pure.cloneElement,
      shallowCompare = pure.shallowCompare,
      Immutable = pure.Immutable,
      Pre = pure.Pre,
      formula = pure.formula,
      defaults = pure.defaults;


  var Grid = (0, _grid2.default)(pure);
  var ZERO_MEASURES = [null, '', 0, '0', '0px'];

  return pure.impure({ displayName: 'Accordion',
    propTypes: { styles: PropTypes.object.isRequired,
      theme: PropTypes.object.isRequired,
      mapHeader: PropTypes.func.isRequired,
      mapContent: PropTypes.func.isRequired,
      data: PropTypes.object.isRequired,
      expandedDocumentID: PropTypes.any,
      transitionDurationMS: PropTypes.number,
      orientation: PropTypes.oneOf(['ttb', 'ltr']),
      autoToggle: PropTypes.bool.isRequired
    },
    defaultProps: (0, _extends3.default)({}, defaults, { orientation: 'ttb',
      autoToggle: true
    }),
    state: { expandedID: null },
    init: function init() {
      var _this = this;

      this.contents = {};
      this.toggleDocument = function (documentID) {
        var node = _this.contents[documentID];
        var orientation = _this.props.orientation;

        if (ZERO_MEASURES.includes(node.style[orientation === 'ltr' ? 'maxWidth' : 'maxHeight'])) {
          _this.expandDocument(documentID);
        } else (0, _raf2.default)(_this.collapseDocuments);
      };
      this.collapseDocuments = function () {
        var orientation = _this.props.orientation;

        var failed = [];
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = (0, _getIterator3.default)((0, _values2.default)(_this.contents)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var node = _step.value;

            if (node) node.style[orientation === 'ltr' ? 'maxWidth' : 'maxHeight'] = 0;
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        _this.setState({ expandedID: null });
      };
      this.expandDocument = function (documentID) {
        _this.collapseDocuments();
        _this.setState({ expandedID: documentID });
        (0, _raf2.default)(_this.updateDimension);
      };

      this.updateDimension = function () {
        return _this.props.orientation === 'ltr' ? _this.updateWidth() : _this.updateHeight();
      };

      this.updateHeight = function () {
        var node = _this.contents[_this.state.expandedID];
        if (node) node.style.maxHeight = '100%'; //`${node.scrollHeight}px`
      };
      this.updateWidth = function () {
        var node = _this.contents[_this.state.expandedID];
        if (node) node.style.maxWidth = node.scrollWidth + 'px';
      };
    },
    componentDidMount: function componentDidMount() {
      var expandedDocumentID = this.props.expandedDocumentID;

      if (typeof expandedDocumentID !== 'undefined') this.expandDocument(expandedDocumentID);
    },
    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
      if (nextProps.data !== this.props.data) (0, _raf2.default)(this.updateDimension);
    },
    render: function render() {
      var _this2 = this;

      var _props = this.props,
          styles = _props.styles,
          theme = _props.theme,
          className = _props.className,
          mapHeader = _props.mapHeader,
          mapContent = _props.mapContent,
          data = _props.data,
          orientation = _props.orientation,
          autoToggle = _props.autoToggle,
          gridProps = (0, _objectWithoutProperties3.default)(_props, ['styles', 'theme', 'className', 'mapHeader', 'mapContent', 'data', 'orientation', 'autoToggle']);

      return React.createElement(Grid, (0, _extends3.default)({}, gridProps, {
        className: (0, _classnames2.default)(styles.accordion, theme.accordion, styles[orientation], theme[orientation], className),
        data: data,
        mapDocument: function mapDocument(_ref) {
          var documentIndex = _ref.documentIndex,
              documentID = _ref.documentID,
              datum = _ref.datum;

          var fns = { toggleDocument: function toggleDocument() {
              return _this2.toggleDocument(documentID);
            },
            collapseDocuments: function collapseDocuments() {
              return _this2.collapseDocuments();
            },
            expandDocument: function expandDocument() {
              return _this2.expandDocument(documentID);
            }
          };
          var header = mapHeader({ documentIndex: documentIndex, documentID: documentID, datum: datum.get('header'), fns: fns });

          return React.createElement(
            'div',
            { className: (0, _classnames2.default)(styles.accordionDocument, theme.accordionDocument) },
            React.createElement(
              'button',
              {
                onClick: autoToggle ? fns.toggleDocument : null,
                className: (0, _classnames2.default)(styles.accordionHeader, theme.accordionHeader)
              },
              React.createElement(
                'span',
                null,
                header
              )
            ),
            React.createElement(
              'div',
              {
                ref: function ref(x) {
                  return _this2.contents[documentID] = x;
                },
                className: (0, _classnames2.default)(styles.accordionContent, theme.accordionContent)
              },
              _this2.state.expandedID === documentID && mapContent({ documentIndex: documentIndex, documentID: documentID, datum: datum.get('content'), fns: fns })
            )
          );
        }
      }));
    }
  });
}