'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = column;

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _pane = require('../pane');

var _pane2 = _interopRequireDefault(_pane);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function column(pure) {
  var React = pure.React,
      PropTypes = pure.PropTypes,
      cloneElement = pure.cloneElement,
      formula = pure.formula,
      Pre = pure.Pre,
      defaults = pure.defaults;

  var Pane = (0, _pane2.default)(pure);

  var SortIcon = function SortIcon(_ref) {
    var direction = _ref.direction;

    var sortClass = 'fa fa-sort';
    if (direction === 'asc' || direction === 'desc') sortClass += '-' + direction;
    return React.createElement('i', { className: sortClass });
  };

  return function Column(columnID) {
    var columnProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var calculatedProps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    /** TODO: WRAP IN CONTEXT MAYBE TO KEEP SEPARATE FROM OTHER GRIDS */
    var headerForm = formula('column-header-' + columnID);
    var cellForm = formula('column-cell-' + columnID);
    var footerForm = formula('column-footer-' + columnID);

    var HEADER_CHECKBOX = 'HEADER_CHECKBOX';
    var HEADER_RADIO = 'HEADER_RADIO';

    var getCellName = function getCellName(_ref2) {
      var type = _ref2.type,
          documentID = _ref2.documentID;
      return 'R-' + documentID + '_C-' + columnID + '_T-' + type.toUpperCase();
    };

    var Header = pure({ displayName: 'Column.Header',
      propTypes: { theme: PropTypes.object.isRequired,
        styles: PropTypes.object.isRequired,
        style: PropTypes.object.isRequired,
        className: PropTypes.string,
        fields: PropTypes.object.isRequired
      },
      defaultProps: (0, _extends3.default)({ style: {},
        fields: {}
      }, defaults, columnProps),
      state: { paneVisible: false },
      render: function render() {
        var _this = this;

        var _props = this.props,
            children = _props.children,
            styles = _props.styles,
            theme = _props.theme,
            actions = _props.actions,
            fields = _props.fields,
            paneContent = _props.paneContent;
        var checkbox = fields.checkbox,
            radio = fields.radio,
            sort = fields.sort,
            filter = fields.filter;
        var paneVisible = this.state.paneVisible;

        var paneClassName = paneVisible ? 'paneVisible' : 'paneHidden';
        var className = (0, _classnames2.default)(styles.headerContainer, theme.headerContainer, styles[paneClassName], theme[paneClassName], this.props.className);
        return React.createElement(
          'div',
          { className: className, style: this.props.style },
          React.createElement(
            'span',
            { className: (0, _classnames2.default)(styles.header, theme.header) },
            React.createElement(
              'span',
              { className: (0, _classnames2.default)(styles.headerLeft, theme.headerLeft) },
              checkbox ? React.createElement(headerForm.Field, (0, _extends3.default)({
                name: HEADER_CHECKBOX,
                type: 'checkbox'
              }, checkbox, {
                subscribeForm: { target: cellForm.formName,
                  selectValue: function selectValue(inputs) {
                    return false;
                  }
                },
                shouldUpdate: function shouldUpdate(_ref3) {
                  var currentValue = _ref3.currentValue,
                      subscribed = _ref3.subscribed,
                      subscriptionType = _ref3.subscriptionType,
                      from = _ref3.from;

                  return currentValue === true;
                }
              })) : null,
              React.createElement(
                'span',
                { className: (0, _classnames2.default)(styles.headerContent, theme.headerContent) },
                children
              )
            ),
            React.createElement(
              'span',
              null,
              filter ? React.createElement(
                'button',
                {
                  className: (0, _classnames2.default)(styles.filterButton, theme.filterButton),
                  onClick: function onClick() {
                    return _this.setState({ paneVisible: !paneVisible });
                  }
                },
                React.createElement('i', { className: 'fa fa-filter' })
              ) : null,
              radio ? React.createElement(
                'span',
                null,
                (0, _entries2.default)(radio[0]).map(function (_ref4, i) {
                  var _ref5 = (0, _slicedToArray3.default)(_ref4, 2),
                      name = _ref5[0],
                      text = _ref5[1];

                  return React.createElement('input', { key: i, type: 'radio', name: name, value: text, checked: i === radio[1] });
                })
              ) : null,
              sort && sort.get('cols', []).includes(columnID) ? React.createElement(
                'button',
                { onClick: function onClick() {
                    return actions.sort(columnID);
                  } },
                React.createElement(SortIcon, { direction: sort.getIn(['direction', columnID]) })
              ) : null
            )
          ),
          paneContent ? React.createElement(
            Pane,
            { enabled: paneVisible },
            paneContent
          ) : null
        );
      }
    });

    var Cell = pure({ displayName: 'Column.Cell',
      propTypes: { styles: PropTypes.object.isRequired,
        theme: PropTypes.object.isRequired,
        style: PropTypes.object.isRequired,
        className: PropTypes.string,
        documentID: PropTypes.any.isRequired
      },
      defaultProps: (0, _extends3.default)({ style: {},
        className: ''
      }, defaults, columnProps),
      init: function init() {
        var _this2 = this;

        this.getName = function (type) {
          return getCellName((0, _extends3.default)({ type: type }, _this2.props));
        };
      },
      render: function render() {
        var _props2 = this.props,
            documentID = _props2.documentID,
            checkbox = _props2.checkbox,
            styles = _props2.styles,
            theme = _props2.theme,
            className = _props2.className,
            style = _props2.style,
            children = _props2.children,
            props = (0, _objectWithoutProperties3.default)(_props2, ['documentID', 'checkbox', 'styles', 'theme', 'className', 'style', 'children']);

        return React.createElement(
          'div',
          (0, _extends3.default)({}, props, { className: (0, _classnames2.default)(styles.cellContent, theme.cellContent, className), style: style }),
          checkbox ? React.createElement(
            cellForm.Field,
            {
              name: this.getName('checkbox'),
              type: 'checkbox',
              subscribeInput: [headerForm.formName, HEADER_CHECKBOX],
              shouldUpdate: function shouldUpdate(_ref6) {
                var currentValue = _ref6.currentValue,
                    subscribed = _ref6.subscribed,
                    subscriptionType = _ref6.subscriptionType;

                return true;
              } },
            children
          ) : children
        );
      }
    });

    var Footer = pure({ displayName: 'Column.Footer',
      propTypes: { styles: PropTypes.object.isRequired,
        theme: PropTypes.object.isRequired,
        style: PropTypes.object.isRequired,
        className: PropTypes.string
      },
      defaultProps: (0, _extends3.default)({ style: {},
        className: ''
      }, defaults, columnProps),
      render: function render() {
        var _props3 = this.props,
            style = _props3.style,
            className = _props3.className,
            children = _props3.children,
            props = (0, _objectWithoutProperties3.default)(_props3, ['style', 'className', 'children']);

        return React.createElement(
          'div',
          { className: className, style: style },
          children
        );
      }
    });

    return { Header: Header, Cell: Cell, Footer: Footer };
  };
}